@@ -1,3 +1,21 @@
+/* GDC -- D front-end for GCC
+   Copyright (C) 2004 David Friedman
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
+ 
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+ 
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
+
 #include "config.h"
 #include "system.h"
 #include "coretypes.h"
@@ -43,6 +61,11 @@ builtin_define_with_value (const char *macro, const char *expansion, int is_str)
     /* nothing */
 }
 
+tree
+default_conversion (tree exp)
+{
+    return exp;
+}
 
 tree
 build_function_call (tree function, tree params)
