@@ -128,7 +128,11 @@ static char lang_name[6] = "GNU D";
 #undef LANG_HOOKS_TREE_INLINING_CONVERT_PARM_FOR_INLINING
 #define LANG_HOOKS_TREE_INLINING_CONVERT_PARM_FOR_INLINING d_convert_parm_for_inlining
 #endif
+#endif
 
+#if D_GCC_VER >= 44
+#undef LANG_HOOKS_INIT_TS
+#define LANG_HOOKS_INIT_TS d_init_ts
 #endif
 
 
@@ -149,6 +153,9 @@ static const char * iprefix;
 static bool std_inc; // %%FIX: find a place for this
 static const char * fonly_arg;
 static const char * multilib_dir;
+// Because of PR16888, on x86 platforms, GCC clears unused reg names.
+// As this doesn't affect us, need a way to restore them.
+static const char *saved_reg_names[FIRST_PSEUDO_REGISTER];
 
 static unsigned int
 d_init_options (unsigned int, const char ** argv)
@@ -297,6 +304,12 @@ static bool
 d_init ()
 {
     const char * cpu_versym = NULL;
+
+#if D_GCC_VER >= 40
+    /* Restore register names if any were cleared during backend init */
+    if (memcmp (reg_names, saved_reg_names, sizeof reg_names))
+	memcpy (reg_names, saved_reg_names, sizeof reg_names);
+#endif
     
     /* Currently, isX86_64 indicates a 64-bit target in general and is not
        Intel-specific. */
@@ -404,11 +417,17 @@ d_init ()
     if (TARGET_LONG_DOUBLE_128)
 	VersionCondition::addPredefinedGlobalIdent("GNU_LongDouble128");
 #endif
-    
-    if (d_have_inline_asm() && cpu_versym && strcmp(cpu_versym, "X86") == 0)
+
+    if (d_have_inline_asm())
     {
 	VersionCondition::addPredefinedGlobalIdent("D_InlineAsm");
-	VersionCondition::addPredefinedGlobalIdent("D_InlineAsm_X86");
+
+	if (cpu_versym && strcmp(cpu_versym, "X86") == 0)
+	    VersionCondition::addPredefinedGlobalIdent("D_InlineAsm_X86");
+	// TODO: D_InlineAsm_X86_64
+
+	/* Should define this anyway to set us apart from the competition. */
+	VersionCondition::addPredefinedGlobalIdent("GNU_InlineAsm");
     }
 
     /* Setting global.params.cov forces module info generation which is
@@ -715,8 +734,10 @@ d_handle_option (size_t scode, const char *arg, int value)
 	  gen.warnSignCompare = value;
 	  break;
       case OPT_fXf_:
+#if V1
 	  global.params.doXGeneration = 1;
 	  global.params.xfilename = (char*)arg;
+#endif
 	  break;
       default:
 	  break;
@@ -730,7 +751,15 @@ bool d_post_options(const char ** fn)
     if (num_in_fnames)
 	*fn = in_fnames[0];
 #if D_GCC_VER >= 40
+    // Save register names for restoring later.
+    memcpy (saved_reg_names, reg_names, sizeof reg_names);
 
+#if D_GCC_VER >= 43
+    // Workaround for embedded functions, don't inline if debugging is on.
+    flag_inline_small_functions = !write_symbols;
+#endif
+
+#if D_GCC_VER < 44
     // Inline option code copied from c-opts.c
     flag_inline_trees = 1;
 
@@ -739,6 +768,7 @@ bool d_post_options(const char ** fn)
 	flag_no_inline = 1;
     if (flag_inline_functions)
 	flag_inline_trees = 2;
+#endif
 
     /* If we are given more than one input file, we must use
        unit-at-a-time mode.  */
@@ -1796,7 +1826,7 @@ tree_code_type[] = {
 #include "d/d-tree.def"
 };
 
-#else
+#elif D_GCC_VER < 44
 
 const enum tree_code_class
 tree_code_type[] = {
@@ -1808,6 +1838,8 @@ tree_code_type[] = {
 
 #undef DEFTREECODE
 
+#if D_GCC_VER < 44
+
 /* Table indexed by tree code giving number of expression
  operands beyond the fixed part of the node structure.
  Not used for types or decls. */
@@ -1832,6 +1864,8 @@ const char *const tree_code_name[] = {
 };
 #undef DEFTREECODE
 
+#endif
+
 #if D_GCC_VER >= 40 && D_GCC_VER < 43
 
 static void
@@ -1879,14 +1913,23 @@ d_types_compatible_p (tree t1, tree t2)
 	|| (t2 == d_va_list && t1 == va_list_type_node))
 	return 1;
 
-    /* Is compatible if aggregates are same type and share the same
+    /* Is compatible if aggregates are same type or share the same
        attributes. The frontend should have already ensured that types
        aren't wildly different anyway... */
-    if (AGGREGATE_TYPE_P (t1) && AGGREGATE_TYPE_P (t2)
-	&& TREE_CODE (t1) == TREE_CODE (t2)
-	&& TYPE_ATTRIBUTES (t1) == TYPE_ATTRIBUTES (t2))
-	return 1;
+    if (AGGREGATE_TYPE_P (t1) && AGGREGATE_TYPE_P (t2) &&
+	TREE_CODE (t1) == TREE_CODE (t2))
+    {
+	tree type1 = TREE_TYPE (t1);
+	tree type2 = TREE_TYPE (t2);
 
+	if (TREE_CODE (t1) == ARRAY_TYPE
+	    || TREE_CODE (t2) == ARRAY_TYPE) {
+	    return (TREE_CODE (type1) == TREE_CODE (type2));
+	}
+	
+	return (TYPE_ATTRIBUTES (t1) == TYPE_ATTRIBUTES (t2));
+    }
+    /* else */
     return 0;
 }
 
@@ -1915,8 +1958,15 @@ d_convert_parm_for_inlining  (tree parm, tree value, tree fndecl, int argnum)
 }
 #endif
     
-#endif
+#endif // D_GCC_VER < 44
 
+#if D_GCC_VER >= 44
+static void
+d_init_ts (void)
+{
+    tree_contains_struct[STATIC_CHAIN_DECL][TS_DECL_COMMON] = 1;
+}
+#endif
 
 struct lang_type *
 build_d_type_lang_specific(Type * t)
